==> sip_script_runner-0.0.12.tar.gz: lib/domain/script.dart <==
import 'package:equatable/equatable.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:sip_script_runner/domain/scripts_config.dart';
import 'package:sip_script_runner/utils/constants.dart';

part 'script.g.dart';

/// Parses an entry from the `scripts.yaml` file
@JsonSerializable(constructor: 'defaults')
class Script extends Equatable {
  const Script({
    required this.name,
    required this.commands,
    required this.aliases,
    required this.description,
    required this.scripts,
    required this.bail,
  });

  const Script.defaults({
    required this.name,
    this.commands = const [],
    this.aliases = const {},
    this.scripts,
    this.description,
    this.bail = false,
  });

  factory Script.fromJson(String name, dynamic json) {
    final possibleCommands = _tryReadListOrString(json);

    if (possibleCommands != null) {
      return Script.defaults(name: name, commands: possibleCommands);
    }

    return _$ScriptFromJson(
      {
        ...?json,
        'name': name,
      },
    );
  }

  final String name;

  @JsonKey(readValue: _readCommand)
  final List<String> commands;

  @JsonKey(
    name: Keys.aliases,
    readValue: _retrieveStrings,
  )
  final Set<String> aliases;
  @JsonKey(
    name: Keys.bail,
    readValue: _retrieveBool,
  )
  final bool bail;
  @JsonKey(name: Keys.description)
  final String? description;

  @JsonKey(readValue: _readScriptsConfig)
  final ScriptsConfig? scripts;

  Map<String, dynamic> toJson() => _$ScriptToJson(this);

  String listOut({
    StringBuffer? buffer,
    String? prefix,
    String Function(String)? wrapCallableKey,
    String Function(String)? wrapNonCallableKey,
    String Function(String)? wrapMeta,
  }) {
    buffer ??= StringBuffer();

    if (name.startsWith('_')) return buffer.toString();

    wrapCallableKey ??= (key) => key;
    wrapNonCallableKey ??= (key) => key;
    wrapMeta ??= (meta) => meta;
    prefix ??= '';

    if (description != null) {
      buffer.writeln('${prefix}${wrapMeta(Keys.description)}: $description');
    }

    if (aliases.isNotEmpty) {
      buffer
          .writeln('${prefix}${wrapMeta(Keys.aliases)}: ${aliases.join(', ')}');
    }

    scripts?.listOut(
      buffer: buffer,
      prefix: '$prefix  ',
      wrapCallableKey: wrapCallableKey,
      wrapNonCallableKey: wrapNonCallableKey,
      wrapMeta: wrapMeta,
    );

    return buffer.toString();
  }

  @override
  List<Object?> get props => _$props;
}

bool? _retrieveBool(Map json, String key) {
  final value = json[key];
  if (value is bool) {
    return value;
  }

  if (value == null && json.containsKey(key)) {
    return true;
  }

  if (value is String) {
    return switch (value.toLowerCase()) {
      'true' => true,
      'yes' => true,
      'y' => true,
      _ => false,
    };
  }

  return null;
}

List? _retrieveStrings(Map json, String key) {
  return _tryReadListOrString(json[key]);
}

Map? _readScriptsConfig(Map json, String key) {
  final mutableMap = {...json};

  // remove all other keys
  mutableMap.removeWhere(
    (key, _) => {
      ...Keys.values,
      'name',
    }.contains(key),
  );

  if (mutableMap.isEmpty) {
    return null;
  }

  return mutableMap;
}

List<String>? _readCommand(Map json, String key) {
  return _tryReadListOrString(json[key]) ??
      _tryReadListOrString(json[Keys.command]);
}

List<String>? _tryReadListOrString(dynamic json) {
  if (json is String) {
    final trimmed = json.trim();
    if (trimmed.isEmpty) return null;

    return [trimmed];
  } else if (json is List) {
    final list = <String>[];
    for (final e in json) {
      if (e is! String) continue;

      final trimmed = e.trim();
      if (trimmed.isEmpty) continue;

      list.add(trimmed);
    }

    return list;
  }

  return null;
}

==> sip_script_runner-0.1.0.tar.gz: lib/domain/script.dart <==
import 'package:equatable/equatable.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:sip_script_runner/domain/scripts_config.dart';
import 'package:sip_script_runner/utils/constants.dart';

part 'script.g.dart';

/// Parses an entry from the `scripts.yaml` file
@JsonSerializable(constructor: 'defaults')
class Script extends Equatable {
  const Script({
    required this.name,
    required this.commands,
    required this.aliases,
    required this.description,
    required this.scripts,
    required this.bail,
    required this.parents,
  });

  const Script.defaults({
    required this.name,
    this.commands = const [],
    this.aliases = const {},
    this.scripts,
    this.description,
    this.bail = false,
    this.parents,
  });

  factory Script.fromJson(String name, dynamic json, {List<String>? parents}) {
    final possibleCommands = _tryReadListOrString(json);

    if (possibleCommands != null) {
      return Script.defaults(
        name: name,
        commands: possibleCommands,
        parents: parents,
      );
    }

    return _$ScriptFromJson(
      {
        ...?json,
        Keys.name: name,
        if (parents != null) Keys.parents: parents,
      },
    );
  }

  @JsonKey(name: Keys.name)
  final String name;

  @JsonKey(readValue: _readCommand)
  final List<String> commands;

  @JsonKey(name: Keys.parents)
  final List<String>? parents;

  @JsonKey(
    name: Keys.aliases,
    readValue: _retrieveStrings,
  )
  final Set<String> aliases;
  @JsonKey(
    name: Keys.bail,
    readValue: _retrieveBool,
  )
  final bool bail;
  @JsonKey(name: Keys.description)
  final String? description;

  @JsonKey(readValue: _readScriptsConfig)
  final ScriptsConfig? scripts;

  Map<String, dynamic> toJson() => _$ScriptToJson(this);

  String listOut({
    StringBuffer? buffer,
    String? prefix,
    String Function(String)? wrapCallableKey,
    String Function(String)? wrapNonCallableKey,
    String Function(String)? wrapMeta,
  }) {
    buffer ??= StringBuffer();

    if (name.startsWith('_')) return buffer.toString();

    wrapCallableKey ??= (key) => key;
    wrapNonCallableKey ??= (key) => key;
    wrapMeta ??= (meta) => meta;
    prefix ??= '';

    if (description != null) {
      buffer.writeln('${prefix}${wrapMeta(Keys.description)}: $description');
    }

    if (aliases.isNotEmpty) {
      buffer
          .writeln('${prefix}${wrapMeta(Keys.aliases)}: ${aliases.join(', ')}');
    }

    scripts?.listOut(
      buffer: buffer,
      prefix: '$prefix  ',
      wrapCallableKey: wrapCallableKey,
      wrapNonCallableKey: wrapNonCallableKey,
      wrapMeta: wrapMeta,
    );

    return buffer.toString();
  }

  @override
  List<Object?> get props => _$props;
}

bool? _retrieveBool(Map json, String key) {
  final value = json[key];
  if (value is bool) {
    return value;
  }

  if (value == null && json.containsKey(key)) {
    return true;
  }

  if (value is String) {
    return switch (value.toLowerCase()) {
      'true' => true,
      'yes' => true,
      'y' => true,
      _ => false,
    };
  }

  return null;
}

List? _retrieveStrings(Map json, String key) {
  return _tryReadListOrString(json[key]);
}

Map? _readScriptsConfig(Map json, String key) {
  final parents = [...(json[Keys.parents] as List<String>? ?? [])];
  final name = json[Keys.name] as String;

  final mutableMap = {...json};

  final removeKeys = {...Keys.values};
  // remove all other keys
  mutableMap.removeWhere(
    (key, _) => removeKeys.contains(key),
  );

  if (mutableMap.isEmpty) {
    return null;
  }

  parents.add(name);
  mutableMap[Keys.parents] = parents;

  return mutableMap;
}

List<String>? _readCommand(Map json, String key) {
  return _tryReadListOrString(json[key]) ??
      _tryReadListOrString(json[Keys.command]);
}

List<String>? _tryReadListOrString(dynamic json) {
  if (json is String) {
    final trimmed = json.trim();
    if (trimmed.isEmpty) return null;

    return [trimmed];
  } else if (json is List) {
    final list = <String>[];
    for (final e in json) {
      if (e is! String) continue;

      final trimmed = e.trim();
      if (trimmed.isEmpty) continue;

      list.add(trimmed);
    }

    return list;
  }

  return null;
}
